@@ -10,9 +10,6 @@ from numerix import nx
 LUTSIZE = rcParams['image.lut']
 
 
-_cool_data = {'red':   ((),()),
-              'green': ((),()),
-              'blue':  ((),())}
               
 _bone_data = {'red':   ((0., 0., 0.),(1.0, 1.0, 1.0)),
               'green': ((0., 0., 0.),(1.0, 1.0, 1.0)),
@@ -386,13 +383,6 @@ def get_cmap(name=None, lut=None):
 
 # These are provided for backwards compat
 import sys
-def ColormapJet(N=LUTSIZE):
-    verbose.report_error("ColormapJet deprecated, please use cm.jet instead")
-    return colors.LinearSegmentedColormap('jet',  _jet_data, N)
-
-def Grayscale(N=LUTSIZE):
-    verbose.report_error("Grayscale deprecated, please use cm.gray instead")
-    return colors.LinearSegmentedColormap('gray',  _gray_data, N)
 
 
 class ScalarMappable:    
@@ -422,15 +412,22 @@ class ScalarMappable:
 
     def to_rgba(self, x, alpha=1.0):
         # assume normalized rgb, rgba
+        #print '0', type(x), x.shape
         x = asarray(x)
+        #print '1', type(x), x.shape
         if len(x.shape)>2: return x
         x = self.norm(x)
-        return self.cmap(x, alpha)
+        x = self.cmap(x, alpha)
+        return x
     
     def set_array(self, A):
         'Set the image array from numeric/numarray A'
         self._A = A.astype(nx.Float32)
 
+    def get_array(self):
+        'Return the array'
+        return self._A 
+
     def set_clim(self, vmin=None, vmax=None):
         'set the norm limits for image scaling'
         self.norm.vmin = vmin
@@ -447,7 +444,7 @@ class ScalarMappable:
         self.changed()
         
     def set_norm(self, norm):
-        'set the colormap for luminance data'
+        'set the normalization instance'
         if norm is None: norm = colors.normalize()
         self.norm = norm
         self.changed()
@@ -471,7 +468,7 @@ class ScalarMappable:
         cmap of another image
         """
         self.observers.append(mappable)
-        
+
     def notify(self, mappable):
         """
         If this is called then we are pegged to another mappable.
