@@ -1,52 +1,44 @@
 from __future__ import division
 
-import os, math
+import math
+import os
 import sys
+import warnings
 def fn_name(): return sys._getframe(1).f_code.co_name
 
-import matplotlib
-from matplotlib import verbose
-from matplotlib.numerix import asarray, fromstring, UInt8, zeros, \
-     where, transpose, nonzero, indices, ones, nx
-
-import matplotlib.numerix as numerix
-from matplotlib.cbook import is_string_like, enumerate, onetrue
-from matplotlib.font_manager import fontManager
-
-from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
-     FigureManagerBase, FigureCanvasBase, error_msg
-from matplotlib._pylab_helpers import Gcf
-from matplotlib.figure import Figure
-from matplotlib.mathtext import math_parse_s_ft2font
-
-pygtk_version_required = (1,99,16)
-try:
-    import pygtk
-    if not matplotlib.FROZEN:
-        pygtk.require('2.0')
-except:
-    print >> sys.stderr, sys.exc_info()[1]
-    raise SystemExit('PyGTK version %d.%d.%d or greater is required to run the GTK Matplotlib backends'
-                     % pygtk_version_required)
-
-import gtk, gobject, pango
-from gtk import gdk
+import gobject
+import gtk; gdk = gtk.gdk
+import pango
+pygtk_version_required = (2,0,0)
 if gtk.pygtk_version < pygtk_version_required:
     raise SystemExit ("PyGTK %d.%d.%d is installed\n"
                       "PyGTK %d.%d.%d or later is required"
                       % (gtk.pygtk_version + pygtk_version_required))
-backend_version = "%d.%d.%d" % gtk.pygtk_version
 del pygtk_version_required
 
+import matplotlib
+from matplotlib._pylab_helpers import Gcf
+from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
+     FigureManagerBase, FigureCanvasBase
+from matplotlib.cbook import is_string_like, enumerate
+from matplotlib.figure import Figure
+from matplotlib.font_manager import fontManager
+from matplotlib.mathtext import math_parse_s_ft2font
+import matplotlib.numerix as numerix
+from matplotlib.numerix import asarray, fromstring, UInt8, zeros, \
+     where, transpose, nonzero, indices, ones, nx
 
-DEBUG = False
+if numerix.which[0] == "numarray":
+    from matplotlib.backends._na_backend_gdk import pixbuf_get_pixels_array
+else:
+    from matplotlib.backends._nc_backend_gdk import pixbuf_get_pixels_array
 
-# the true dots per inch on the screen; should be display dependent
-# see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
-PIXELS_PER_INCH = 96
+
+backend_version = "%d.%d.%d" % gtk.pygtk_version
+_debug = False
 
 # Image formats that this backend supports - for FileChooser and print_figure()
-IMAGE_FORMAT          = ['eps', 'jpg', 'png', 'ps', 'svg'] + ['bmp'] # , 'raw', 'rgb']
+IMAGE_FORMAT  = ['eps', 'jpg', 'png', 'ps', 'svg'] + ['bmp'] # , 'raw', 'rgb']
 IMAGE_FORMAT.sort()
 IMAGE_FORMAT_DEFAULT  = 'png'
 
@@ -78,15 +70,17 @@ class RendererGDK(RendererBase):
     rotated = {}  # a map from text prop tups to rotated text pixbufs
 
     def __init__(self, gtkDA, dpi):
-        # gtkDA is used in '<widget>.create_pango_layout(s)' only
+        # widget gtkDA is used for:
+        #  '<widget>.create_pango_layout(s)'
+        #  cmap line below)
         self.gtkDA = gtkDA
         self.dpi   = dpi
         self._cmap = gtkDA.get_colormap()
         
-    def _set_pixmap (self, gdkDrawable):
-        self.gdkDrawable = gdkDrawable
+    def set_pixmap (self, pixmap):
+        self.gdkDrawable = pixmap
 
-    def _set_width_height (self, width, height):
+    def set_width_height (self, width, height):
         """w,h is the figure w,h not the pixmap w,h
         """
         self.width, self.height = width, height
@@ -112,37 +106,31 @@ class RendererGDK(RendererBase):
             # set clip rect?
 
         flipud = origin=='lower'
-        rows, cols, s = im.as_str(flipud)
+        rows, cols, image_str = im.as_str(flipud)
 
-        X = fromstring(s, UInt8)
-        X.shape = rows, cols, 4
+        image_array = fromstring(image_str, UInt8)
+        image_array.shape = rows, cols, 4
 
-        pb = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
-                            #has_alpha=1, bits_per_sample=8,
-                            has_alpha=True, bits_per_sample=8,
-                            width=cols, height=rows)
-        try:
-            pa = pb.get_pixels_array()
-        except AttributeError:
-            pa = pb.pixel_array
-        except RuntimeError, exc: #  pygtk was not compiled with Numeric Python support
-            verbose.report_error('Error: %s' % exc)
-            return
+        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
+                                has_alpha=True, bits_per_sample=8,
+                                width=cols, height=rows)
 
-        pa[:,:,:] = X
+        array = pixbuf_get_pixels_array(pixbuf)
+        array[:,:,:] = image_array
 
         gc = self.new_gc()
 
-        if flipud:  y = self.height-y-rows
+        if flipud:
+            y = self.height-y-rows
 
-        try: # requires GTK+ 2.2
+        try: # new in 2.2
             # can use None instead of gc.gdkGC, if don't need clipping
-            self.gdkDrawable.draw_pixbuf (gc.gdkGC, pb, 0, 0,
+            self.gdkDrawable.draw_pixbuf (gc.gdkGC, pixbuf, 0, 0,
                                           int(x), int(y), cols, rows,
                                           gdk.RGB_DITHER_NONE, 0, 0)
         except AttributeError:
-            # is deprecated
-            pb.render_to_drawable(self.gdkDrawable, gc.gdkGC, 0, 0,
+            # deprecated in 2.2
+            pixbuf.render_to_drawable(self.gdkDrawable, gc.gdkGC, 0, 0,
                                   int(x), int(y), cols, rows,
                                   gdk.RGB_DITHER_NONE, 0, 0)
 
@@ -152,7 +140,7 @@ class RendererGDK(RendererBase):
                                    int(x2), self.height-int(y2))
 
 
-    def draw_lines(self, gc, x, y):
+    def draw_lines(self, gc, x, y, transform=None):
         x = x.astype(nx.Int16)
         y = self.height - y.astype(nx.Int16)  
         self.gdkDrawable.draw_lines(gc.gdkGC, zip(x,y))
@@ -188,9 +176,12 @@ class RendererGDK(RendererBase):
     def draw_text(self, gc, x, y, s, prop, angle, ismath):
         x, y = int(x), int(y)
 
-        if angle not in (0,90):
-            verbose.report_error('The GTK backend cannot draw text at a %i degree angle, try GtkAgg instead' % angle)
+        if x <0 or y <0: # window has shrunk and text is off the edge
+            return
 
+        if angle not in (0,90):
+            warnings.warn('backend_gdk: unable to draw text at angles ' +
+                          'other than 0 or 90')
         elif ismath:
             self._draw_mathtext(gc, x, y, s, prop, angle)
 
@@ -213,7 +204,7 @@ class RendererGDK(RendererBase):
             x -= width
         y -= height
         
-        imw, imh, s = fonts[0].image_as_str()
+        imw, imh, image_str = fonts[0].image_as_str()
         N = imw*imh
 
         # a numpixels by num fonts array
@@ -222,8 +213,8 @@ class RendererGDK(RendererBase):
         for i, font in enumerate(fonts):
             if angle == 90:
                 font.horiz_image_to_vert_image() # <-- Rotate
-            imw, imh, s = font.image_as_str()
-            Xall[:,i] = fromstring(s, UInt8)  
+            imw, imh, image_str = font.image_as_str()
+            Xall[:,i] = fromstring(image_str, UInt8)  
 
         # get the max alpha at each pixel
         Xs = numerix.mlab.max(Xall,1)
@@ -231,31 +222,25 @@ class RendererGDK(RendererBase):
         # convert it to it's proper shape
         Xs.shape = imh, imw
 
-        pb=gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
-                          #has_alpha=1, bits_per_sample=8, width=imw, height=imh)
-                          has_alpha=True, bits_per_sample=8, width=imw, height=imh)
+        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, has_alpha=True,
+                                bits_per_sample=8, width=imw, height=imh)
 
-        try:
-            pa = pb.get_pixels_array()
-        except AttributeError:
-            pa = pb.pixel_array
-        except RuntimeError, exc: #  'pygtk was not compiled with Numeric Python support'
-            verbose.report_error('mathtext not supported: %s' % exc)
-            return        
+        array = pixbuf_get_pixels_array(pixbuf)
 
         rgb = gc.get_rgb()
-        pa[:,:,0]=int(rgb[0]*255)
-        pa[:,:,1]=int(rgb[1]*255)
-        pa[:,:,2]=int(rgb[2]*255)
-        pa[:,:,3]=Xs
+        array[:,:,0]=int(rgb[0]*255)
+        array[:,:,1]=int(rgb[1]*255)
+        array[:,:,2]=int(rgb[2]*255)
+        array[:,:,3]=Xs
 
-        try: # requires GTK+ 2.2
+        try: # new in 2.2
             # can use None instead of gc.gdkGC, if don't need clipping
-            self.gdkDrawable.draw_pixbuf (gc.gdkGC, pb, 0, 0,
-                                          int(x), int(y), cols, rows,
+            self.gdkDrawable.draw_pixbuf (gc.gdkGC, pixbuf, 0, 0,
+                                          int(x), int(y), imw, imh,
                                           gdk.RGB_DITHER_NONE, 0, 0)
         except AttributeError:
-            pb.render_to_drawable(self.gdkDrawable, gc.gdkGC, 0, 0,
+            # deprecated in 2.2
+            pixbuf.render_to_drawable(self.gdkDrawable, gc.gdkGC, 0, 0,
                                   int(x), int(y), imw, imh,
                                   gdk.RGB_DITHER_NONE, 0, 0)
             
@@ -265,7 +250,8 @@ class RendererGDK(RendererBase):
         Draw the text rotated 90 degrees, other angles are not supported
         """
         # this function (and its called functions) is a bottleneck
-        # Pango 1.6 supports rotated text, but pygtk 2.4.0 does not yet have wrapper functions
+        # Pango 1.6 supports rotated text, but pygtk 2.4.0 does not yet have
+        # wrapper functions
         # GTK+ 2.6 pixbufs support rotation
 
         gdrawable = self.gdkDrawable
@@ -276,6 +262,9 @@ class RendererGDK(RendererBase):
         x = int(x-h)
         y = int(y-w)
 
+        if x < 0 or y < 0: # window has shrunk and text is off the edge
+            return
+
         key = (x,y,s,angle,hash(prop))
         imageVert = self.rotated.get(key)
         if imageVert != None:
@@ -284,12 +273,11 @@ class RendererGDK(RendererBase):
 
         imageBack = gdrawable.get_image(x, y, w, h)
         imageVert = gdrawable.get_image(x, y, h, w)
-        #imageFlip = gtk.gdk.Image(type=gdk.IMAGE_NORMAL,
         imageFlip = gtk.gdk.Image(type=gdk.IMAGE_FASTEST,
                                   visual=gdrawable.get_visual(),
                                   width=w, height=h)
         if imageFlip == None or imageBack == None or imageVert == None:
-            verbose.report_error("Could not renderer vertical text", s)
+            warnings.warn("Could not renderer vertical text")
             return
         imageFlip.set_colormap(self._cmap)
         for i in range(w):
@@ -313,16 +301,20 @@ class RendererGDK(RendererBase):
         """
         Create a pango layout instance for Text 's' with properties 'prop'.
         Return - pango layout (from cache if already exists)
+
+        Note that pango assumes a logical DPI of 96
+        Ref: pango/fonts.c/pango_font_description_set_size() manual page
         """
         # problem? - cache gets bigger and bigger, is never cleared out
-        # two (not one) layouts are created for every text item s (then they are cached) - why?
+        # two (not one) layouts are created for every text item s (then they
+        # are cached) - why?
         
         key = self.dpi.get(), s, hash(prop)
         value = self.layoutd.get(key)
         if value != None:
             return value
 
-        size = prop.get_size_in_points() * self.dpi.get() / PIXELS_PER_INCH
+        size = prop.get_size_in_points() * self.dpi.get() / 96.0
         size = round(size)
 
         font_str = '%s, %s %i' % (prop.get_name(), prop.get_style(), size,)
@@ -343,7 +335,6 @@ class RendererGDK(RendererBase):
         return True
 
     def get_canvas_width_height(self):
-        #return self.gtkDA.allocation.width, self.gtkDA.allocation.height        
         return self.width, self.height
 
     def get_text_width_height(self, s, prop, ismath):
@@ -361,7 +352,7 @@ class RendererGDK(RendererBase):
 
 
     def points_to_pixels(self, points):
-        return points * PIXELS_PER_INCH/72.0 * self.dpi.get()/72.0
+        return points/72.0 * self.dpi.get()
 
 
 class GraphicsContextGDK(GraphicsContextBase):
@@ -402,6 +393,9 @@ class GraphicsContextGDK(GraphicsContextBase):
             return color
 
 
+    #def set_antialiased(self, b):
+        # anti-aliasing is not supported by GDK
+
     def set_capstyle(self, cs):
         GraphicsContextBase.set_capstyle(self, cs)
         self.gdkGC.cap_style = self._capd[self._capstyle]
@@ -458,52 +452,25 @@ def new_figure_manager(num, *args, **kwargs):
     canvas  = FigureCanvasGDK(thisFig)
     manager = FigureManagerBase(canvas, num)
     # equals:
-    #manager = FigureManagerBase (FigureCanvasGDK (Figure(*args, **kwargs), num)
+    #manager = FigureManagerBase (FigureCanvasGDK (Figure(*args, **kwargs),
+    #                             num)
     return manager
 
 
-class FigureCanvasGDK(FigureCanvasBase):
+class FigureCanvasGDK (FigureCanvasBase):
     def __init__(self, figure):
         FigureCanvasBase.__init__(self, figure)
-        self._pixmap_width  = -1
-        self._pixmap_height = -1
         
         self._renderer_init()
 
     def _renderer_init(self):
-        #self._renderer = RendererGDK (self, self.figure.dpi) # self is no longer a widget subclass
         self._renderer = RendererGDK (gtk.DrawingArea(), self.figure.dpi)
 
-        
-    def _render_to_pixmap(self, width, height):
-        """Render the figure to a gdk.Pixmap, is used for
-           - rendering the pixmap to display        (pylab.draw)
-           - rendering the pixmap to save to a file (pylab.savefig)
-        Should not be overridden
-        """
-        if DEBUG: print 'FigureCanvasGDK.%s' % fn_name()
-        create_pixmap = False
-        if width > self._pixmap_width:
-            # increase the pixmap in 10%+ (rather than 1 pixel) steps
-            self._pixmap_width  = max (int (self._pixmap_width  * 1.1), width)
-            create_pixmap = True
-
-        if height > self._pixmap_height:
-            self._pixmap_height = max (int (self._pixmap_height * 1.1), height)
-            create_pixmap = True
-
-        if create_pixmap:
-            if DEBUG: print 'FigureCanvasGTK.%s new pixmap' % fn_name()
-            #self._pixmap = gtk.gdk.Pixmap (self.window, self._pixmap_width,
-            self._pixmap = gtk.gdk.Pixmap (None, self._pixmap_width,
-                                           self._pixmap_height, depth=24)
-            # gtk backend must use self.window
-            self._renderer._set_pixmap (self._pixmap)
-
-        self._renderer._set_width_height (width, height)
+    def _render_figure(self, pixmap, width, height):
+        self._renderer.set_pixmap (pixmap)
+        self._renderer.set_width_height (width, height)
         self.figure.draw (self._renderer)
 
-
     def print_figure(self, filename, dpi=150, facecolor='w', edgecolor='w',
                      orientation='portrait'):
         root, ext = os.path.splitext(filename)       
@@ -518,23 +485,21 @@ class FigureCanvasGDK(FigureCanvasBase):
 
         ext = ext.lower()
         if ext in ('jpg', 'png'):          # native printing
-            width, height = self.figure.get_width_height()
-            width, height = int(width), int(height)
-            self._render_to_pixmap(width, height)
+            width, height = self.get_width_height()
+            pixmap = gtk.gdk.Pixmap (None, width, height, depth=24)
+            self._render_figure(pixmap, width, height)
 
-            # jpg colors don't match the display very well, png colors match better
+            # jpg colors don't match the display very well, png colors match
+            # better
             pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8,
                                     width, height)
-            #pixbuf.get_from_drawable(self._pixmap, self._pixmap.get_colormap(),
-            pixbuf.get_from_drawable(self._pixmap, self._renderer._cmap,
+            pixbuf.get_from_drawable(pixmap, pixmap.get_colormap(),
                                      0, 0, 0, 0, width, height)
         
             # pixbuf.save() recognises 'jpeg' not 'jpg'
             if ext == 'jpg': ext = 'jpeg' 
-            try:
-                pixbuf.save(filename, ext)
-            except gobject.GError, exc:
-                error_msg('Save figure failure:\n%s' % (exc,), parent=self)
+
+            pixbuf.save(filename, ext)
 
         elif ext in ('eps', 'ps', 'svg',):
             if ext == 'svg':
@@ -542,27 +507,17 @@ class FigureCanvasGDK(FigureCanvasBase):
             else:
                 from backend_ps  import FigureCanvasPS  as FigureCanvas
 
-            try:
-                fc = self.switch_backends(FigureCanvas)
-                fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
-            except IOError, exc:
-                error_msg("Save figure failure:\n%s: %s" %
-                          (exc.filename, exc.strerror), parent=self)
-            except Exception, exc:
-                error_msg("Save figure failure:\n%s" % exc, parent=self)
-
+            
+            fc = self.switch_backends(FigureCanvas)
+            fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
         elif ext in ('bmp', 'raw', 'rgb',):
-            try: 
-                from backend_agg import FigureCanvasAgg  as FigureCanvas
-            except:
-                error_msg('Save figure failure:\n'
-                          'Agg must be loaded to save as bmp, raw and rgb',
-                          parent=self)                
-            else:
-                fc = self.switch_backends(FigureCanvas)
-                fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
+            
+            from backend_agg import FigureCanvasAgg  as FigureCanvas
+            fc = self.switch_backends(FigureCanvas)
+            fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
 
         else:
-            error_msg('Format "%s" is not supported.\nSupported formats are %s.' %
-                      (ext, ', '.join(IMAGE_FORMAT)),
-                      parent=self)
+            raise ValueError('Format "%s" is not supported.\nSupported formats are %s.' %
+                             (ext, ', '.join(IMAGE_FORMAT)))
+
+        self.figure.set_canvas(self)
