@@ -25,19 +25,20 @@ KNOWN ISSUES
   - update_fonts not implemented
 
 
-Authors   : John Hunter <jdhunter@nitace.bsd.uchicago.edu>
+Authors   : John Hunter <jdhunter@ace.bsd.uchicago.edu>
             Paul Barrett <Barrett@STScI.Edu>
-Copyright : John Hunter (2004), Paul Barrett (2004)
+Copyright : John Hunter (2004,2005), Paul Barrett (2004,2005)
 License   : matplotlib license (PSF compatible)
             The font directory code is from ttfquery,
             see license/LICENSE_TTFQUERY.
 """
 
-import os, sys, glob
+import os, sys, glob, warnings
 import matplotlib
 from matplotlib import afm
 from matplotlib import ft2font
-from matplotlib import rcParams, get_data_path, get_home
+from matplotlib import rcParams, get_data_path, get_home, get_configdir
+
 
 verbose = matplotlib.verbose
 
@@ -64,14 +65,19 @@ X11FontDirectories  = [
     # common application, not really useful
     "/usr/lib/openoffice/share/fonts/truetype/",
     # documented as a good place to install new fonts...
-    "/usr/share/fonts/",
-    # okay, now the OS X variants...
-    "~/Library/Fonts/",
+    "/usr/share/fonts/"]
+    
+OSXFontDirectories = [
     "/Library/Fonts/",
     "/Network/Library/Fonts/",
-    "/System/Library/Fonts/",
-    "System Folder:Fonts:"]
+    "/System/Library/Fonts/"
+]
 
+home = os.environ.get('HOME')
+if home is not None:
+    # user fonts on OSX
+    path = os.path.join(home, 'Library', 'Fonts')
+    OSXFontDirectories.append(path)
 
 def win32FontDirectory():
     """Return the user-specified font directory for Win32."""
@@ -110,17 +116,52 @@ returned by default with AFM fonts as an option.
             return glob.glob(os.path.join(directory, '*.'+fontext))
         try:
             for j in range(_winreg.QueryInfoKey(local)[1]):
-                key, direc, any = _winreg.EnumValue( local, j)
-                if not os.path.dirname(direc):
-                    direc = os.path.join(directory, direc)
-                direc = os.path.abspath(direc).lower()
-                if direc[-4:] == '.'+fontext:
-                    items[direc] = 1
+                try:
+                    key, direc, any = _winreg.EnumValue( local, j)
+                    if not os.path.dirname(direc):
+                        direc = os.path.join(directory, direc)
+                    direc = os.path.abspath(direc).lower()
+                    if direc[-4:] == '.'+fontext:
+                        items[direc] = 1
+                except EnvironmentError:
+                    continue
+                except WindowsError:
+                    continue
+                        
             return items.keys()
         finally:
             _winreg.CloseKey(local)
     return None
 
+def OSXFontDirectory():
+    """Return the system font directories for OS X."""
+
+    fontpaths = []
+    def add(arg,directory,files):
+        fontpaths.append(directory)
+    for fontdir in OSXFontDirectories:
+        try:
+            if os.path.isdir(fontdir):
+                os.path.walk(fontdir, add, None)
+        except (IOError, OSError, TypeError, ValueError):
+            pass
+    return fontpaths
+
+def OSXInstalledFonts(directory=None, fontext=None):
+    """Get list of font files on OS X - ignores font suffix by default"""
+    if directory is None:
+        directory = OSXFontDirectory()
+    
+    files = []
+    for path in directory:
+        if fontext is None:
+            files.extend(glob.glob(os.path.join(path,'*')))
+        else:
+            files.extend(glob.glob(os.path.join(path, '*.'+fontext)))
+            files.extend(glob.glob(os.path.join(path, '*.'+fontext.upper())))
+    return files
+
+
 def x11FontDirectory():
     """Return the system font directories for X11."""
 
@@ -164,10 +205,17 @@ with AFM fonts as an option.
                     fontfiles[f] = 1
         else:
             fontpaths = x11FontDirectory()
+            # check for OS X & load its fonts if present
+            if sys.platform == 'darwin':
+                for f in OSXInstalledFonts():
+                    fontfiles[f] = 1
+
     elif isinstance(fontpaths, (str, unicode)):
         fontpaths = [fontpaths]
     for path in fontpaths:
-        for fname in glob.glob(os.path.join(path, '*.'+fontext)):
+        files = glob.glob(os.path.join(path, '*.'+fontext))
+        files.extend(glob.glob(os.path.join(path, '*.'+fontext.upper())))
+        for fname in files:
 
             fontfiles[os.path.abspath(fname)] = 1
     return [fname for fname in fontfiles.keys() if os.path.exists(fname)]
@@ -368,27 +416,18 @@ the FontKey properties.  If a font property has no dictionary, then
 create it.
 """
 
-    if not fontdict.has_key(prop.name):
-        fontdict[prop.name] = {}
-    fontdict = fontdict[prop.name]
-    if not fontdict.has_key(prop.style):
-        fontdict[prop.style] = {}
-    fontdict = fontdict[prop.style]
-    if not fontdict.has_key(prop.variant):
-        fontdict[prop.variant] = {}
-    fontdict = fontdict[prop.variant]
-    if not fontdict.has_key(prop.weight):
-        fontdict[prop.weight] = {}
-    fontdict = fontdict[prop.weight]
-    if not fontdict.has_key(prop.stretch):
-        fontdict[prop.stretch] = {}
-    fontdict = fontdict[prop.stretch]
-
     try:
         size = str(float(prop.size))
     except ValueError:
         size = prop.size
-    fontdict[size] = fname
+
+    d = fontdict.                    \
+        setdefault(prop.name,    {}).\
+        setdefault(prop.style,   {}).\
+        setdefault(prop.variant, {}).\
+        setdefault(prop.weight,  {}).\
+        setdefault(prop.stretch, {})
+    d[size] = fname
 
 
 def createFontDict(fontfiles, fontext='ttf'):
@@ -401,23 +440,26 @@ dictionary can optionally be created.
     #  Add fonts from list of known font files.
     seen = {}
     for fpath in fontfiles:
+        verbose.report('createFontDict: %s' % (fpath), 'debug')
         fname = fpath.split('/')[-1]
         if seen.has_key(fname):  continue
         else: seen[fname] = 1
-        if fontext == 'ttf':
-            try:
-                font = ft2font.FT2Font(str(fpath))
-            except RuntimeError:
-                verbose.report_error("Could not open font file %s"%fpath)
-                continue
-            prop = ttfFontProperty(font)
-        elif fontext == 'afm':
+        if fontext == 'afm':
             try:
                 font = afm.AFM(file(fpath))
             except RuntimeError:
-                verbose.report_error("Could not open font file %s"%fpath)
+                warnings.warn("Could not open font file %s"%fpath)
                 continue
             prop = afmFontProperty(font)
+        else:
+            #try:
+            font = ft2font.FT2Font(str(fpath))
+            #except RuntimeError:
+            #    warnings.warn("Could not open font file %s"%fpath)
+            #    continue
+            try: prop = ttfFontProperty(font)
+            except: continue
+            
         add_filename(fontdict, prop, fpath)
 
         #  !!!!  Default font algorithm needs improvement
@@ -647,7 +689,7 @@ set_family() method.
         #  Remove family names from list of font names.
         for name in names[:]:
             if name.lower() in font_family:
-                verbose.report_error( msg % name)
+                warnings.warn( msg % name)
                 while name in names:
                     names.remove(name.lower())
 
@@ -754,7 +796,9 @@ font dictionary can act like a font cache.
         #  Load TrueType fonts and create font dictionary.
         
         self.ttffiles = findSystemFonts(paths) + findSystemFonts()
+
         for fname in self.ttffiles:
+            verbose.report('trying fontname %s' % fname, 'debug')
             if fname.lower().find('vera.ttf')>=0:
                 self.defaultFont = fname
                 break
@@ -765,10 +809,17 @@ font dictionary can act like a font cache.
         cache_message = \
 """Saving TTF font cache for non-PS backends to %s.
 Delete this file to have matplotlib rebuild the cache."""
+
         
-        ttfpath = get_home()
-        if ttfpath is None: ttfpath = get_data_path()
-        ttfcache = os.path.join(ttfpath, '.ttffont.cache')
+
+        oldcache = os.path.join(get_home(), 'ttffont.cache')
+        ttfcache = os.path.join(get_configdir(), 'ttffont.cache')
+        if os.path.exists(oldcache):            
+            print >> sys.stderr, 'Moving old ttfcache location "%s" to new location "%s"'%(oldcache, ttfcache)
+            os.rename(oldcache, ttfcache)
+
+        
+
         try:
             import cPickle as pickle
         except ImportError:
@@ -851,9 +902,7 @@ Delete this file to have matplotlib rebuild the cache."""
             verbose.report('findfont returning %s'%fname, 'debug')
             return fname
         
-        if   fontext == 'ttf':
-            fontdict = self.ttfdict
-        elif fontext == 'afm':
+        if fontext == 'afm':
             if len(self.afmdict) == 0:
                 afmpath = os.environ.get('HOME', get_data_path())
                 afmcache = os.path.join(afmpath, '.afmfont.cache')
@@ -868,6 +917,8 @@ Delete this file to have matplotlib rebuild the cache."""
                     pickle.dump(self.afmdict, file(afmcache, 'w'))
                     verbose.report(cache_message % afmcache)
             fontdict = self.afmdict
+        else:
+            fontdict = self.ttfdict
 
         name    = prop.get_family()[0]
         style   = prop.get_style()
@@ -935,7 +986,7 @@ Delete this file to have matplotlib rebuild the cache."""
 
         fontkey = FontKey(name, style, variant, weight, stretch, size)
         add_filename(fontdict, fontkey, self.defaultFont)
-        verbose.report_error('Could not match %s, %s, %s.  Returning %s' % (name, style, variant, self.defaultFont))
+        warnings.warn('Could not match %s, %s, %s.  Returning %s' % (name, style, variant, self.defaultFont))
 
         return self.defaultFont
 
