@@ -70,7 +70,7 @@ from numerix import array, asarray, arange, divide, exp, arctan2, \
 
 
 from numerix.mlab import hanning, cov, diff, svd, rand, std
-from numerix.fft import fft
+from numerix.fft import fft, inverse_fft
 
 from cbook import iterable
 
@@ -130,7 +130,6 @@ def detrend_linear(x):
     a = mean(x) - b*mean(xx)
     return x-(b*xx+a)
 
-
 def psd(x, NFFT=256, Fs=2, detrend=detrend_none,
         window=window_hanning, noverlap=0):
     """
@@ -277,7 +276,7 @@ def cohere(x, y, NFFT=256, Fs=2, detrend=detrend_none,
     """
     
     if len(x)<2*NFFT:
-       verbose.report_error('Coherence is calculated by averaging over NFFT length segments.  Your signal is too short for your choice of NFFT')
+       raise RuntimeError('Coherence is calculated by averaging over NFFT length segments.  Your signal is too short for your choice of NFFT')
     Pxx, f = psd(x, NFFT, Fs, detrend, window, noverlap)
     Pyy, f = psd(y, NFFT, Fs, detrend, window, noverlap)
     Pxy, f = csd(x, y, NFFT, Fs, detrend, window, noverlap)
@@ -367,8 +366,8 @@ def polyfit(x,y,N):
 
     """
 
-    x = asarray(x)
-    y = asarray(y)
+    x = asarray(x)+0.
+    y = asarray(y)+0.
     y = reshape(y, (len(y),1))
     X = Matrix(vander(x, N+1))
     Xt = Matrix(transpose(X))
@@ -395,7 +394,7 @@ def polyval(p,x):
     See also polyfit
     
     """
-
+    x = asarray(x)+0.
     p = reshape(p, (len(p),1))
     X = vander(x,len(p))
     y =  matrixmultiply(X,p)
@@ -617,7 +616,11 @@ def hist(y, bins=10, normed=0):
         if ymin==ymax:
             ymin -= 0.5
             ymax += 0.5
-        bins = linspace(ymin, ymax, bins)
+
+        if bins==1: bins=ymax
+        dy = (ymax-ymin)/bins 
+        bins = ymin + dy*arange(bins)
+
 
     n = searchsorted(sort(y), bins)
     n = diff(concatenate([n, [len(y)]]))
@@ -814,8 +817,8 @@ def rem(x,y):
             return x - y * fix(x/y)
         except OverflowError:
             return None
-    verbose.report_error('Dimension error')
-    return None
+    raise RuntimeError('Dimension error')
+
 
 def norm(x,y=2):
     """
@@ -850,8 +853,7 @@ def norm(x,y=2):
         elif y=='fro':
             return numerix.mlab.sqrt(asum(numerix.mlab.diag(matrixmultiply(transpose(x),x))))
         else:
-            verbose.report_error('Second argument not permitted for matrices')
-            return None
+            raise RuntimeError('Second argument not permitted for matrices')
         
     else:
         if y == 'inf':
@@ -1187,6 +1189,74 @@ def get_sparse_matrix(M,N,frac=0.1):
         data[x,y] = rand()
     return data
 
+def dist(x,y):
+    'return the distance between two points'
+    d = x-y
+    return numerix.mlab.sqrt(dot(d,d))
+
+def dist_point_to_segment(p, s0, s1):
+    """
+    get the distance of a point to a segment.
+
+    p, s0, s1 are xy sequences
+
+    This algorithm from
+    http://softsurfer.com/Archive/algorithm_0102/algorithm_0102.htm#Distance%20to%20Ray%20or%20Segment
+    """
+    p = asarray(p, Float)
+    s0 = asarray(s0, Float)
+    s1 = asarray(s1, Float)    
+    v = s1 - s0
+    w = p - s0
+
+    c1 = dot(w,v);
+    if ( c1 <= 0 ):
+        return dist(p, s0);
+
+    c2 = dot(v,v)
+    if ( c2 <= c1 ):
+        return dist(p, s1);
+
+    b = c1 / c2
+    pb = s0 + b * v;
+    return dist(p, pb)
+
+def segments_intersect(s1, s2):
+    """
+    Return True if s1 and s2 intersect.
+    s1 and s2 are defines as
+
+    s1: (x1, y1), (x2, y2)
+    s2: (x3, y3), (x4, y4)
+
+    """
+    (x1, y1), (x2, y2) = s1
+    (x3, y3), (x4, y4) = s2
+
+    den = ((y4-y3) * (x2-x1)) - ((x4-x3)*(y2-y1))
+
+    n1 = ((x4-x3) * (y1-y3)) - ((y4-y3)*(x1-x3))
+    n2 = ((x2-x1) * (y1-y3)) - ((y2-y1)*(x1-x3))
+
+    if den == 0:
+        # lines parallel
+        return False
+
+    u1 = n1/den
+    u2 = n2/den
+
+    return 0.0 <= u1 <= 1.0 and 0.0 <= u2 <= 1.0
+
+
+def fftsurr(x):
+    """
+    Compute an FFT phase randomized surrogate of x
+    """
+    z = fft(x)
+    a = 2.*pi*1j
+    phase = a*rand(len(x))
+    z = z*exp(phase)
+    return inverse_fft(z).real
 
 ### the following code was written and submitted by Fernando Perez
 ### from the ipython numutils package under a BSD license
