@@ -190,15 +190,13 @@ John D. Hunter <jdhunter@ace.bsd.uhicago.edu>
 Most of the other commands are from Numeric, MLab and FFT, with the
 exception of those in mlab.py provided by matplotlib.
 """
-import warnings
+import sys, warnings
 import cm
 import _pylab_helpers
 import mlab  #so I can override hist, psd, etc...
 
 from axes import Axes, PolarAxes
 import backends
-from backends import new_figure_manager, draw_if_interactive, show
-
 from cbook import flatten, is_string_like, exception_to_str, popd, \
      silent_list, iterable, enumerate
 from colors import normalize
@@ -210,6 +208,13 @@ from backend_bases import FigureCanvasBase
 from artist import ArtistInspector, getp, get
 from artist import setp as _setp
 
+# a hack to keep old versions of ipython working with mpl after bug
+# fix #1209354
+if 'IPython.Shell' in  sys.modules:
+    from backends import new_figure_manager, draw_if_interactive, show
+else:
+    from backends import pylab_setup
+    new_figure_manager, draw_if_interactive, show = pylab_setup()
 
 
 
@@ -304,14 +309,14 @@ def _shift_string(s):
 def colorbar(tickfmt='%1.1f', cax=None, orientation='vertical'):
     """
     Create a colorbar for current image
-    
+
     tickfmt is a format string to format the colorbar ticks
 
     cax is a colorbar axes instance in which the colorbar will be
     placed.  If None, as default axesd will be created resizing the
     current aqxes to make room for it.  If not None, the supplied axes
     will be used and the other axes positions will be unchanged.
-    
+
     orientation is the colorbar orientation: one of 'vertical' | 'horizontal'
     return value is the colorbar axes instance
     """
@@ -319,7 +324,7 @@ def colorbar(tickfmt='%1.1f', cax=None, orientation='vertical'):
     ret = gcf().colorbar(mappable, tickfmt, cax, orientation)
     draw_if_interactive()
     return ret
-    
+
 def colors():
     """
     This is a do nothing function to provide you with help on how
@@ -472,12 +477,12 @@ def get_current_fig_manager():
 def connect(s, func):
     return get_current_fig_manager().canvas.mpl_connect(s, func)
 if FigureCanvasBase.mpl_connect.__doc__ is not None:
-    connect.__doc__ = FigureCanvasBase.mpl_connect.__doc__
+    connect.__doc__ = _shift_string(FigureCanvasBase.mpl_connect.__doc__)
 
 def disconnect(cid):
     return get_current_fig_manager().canvas.mpl_disconnect(cid)
 if FigureCanvasBase.mpl_disconnect.__doc__ is not None:
-    disconnect.__doc__ = FigureCanvasBase.mpl_disconnect.__doc__
+    disconnect.__doc__ = _shift_string(FigureCanvasBase.mpl_disconnect.__doc__)
 
 def get_plot_commands(): return ( 'axes', 'axis', 'bar', 'boxplot', 'cla', 'clf',
     'close', 'colorbar', 'cohere', 'csd', 'draw', 'errorbar',
@@ -498,7 +503,6 @@ def raise_msg_to_str(msg):
 
 
 
-
 def axis(*v, **kwargs):
     """
     Set/Get the axis properties::
@@ -513,6 +517,10 @@ def axis(*v, **kwargs):
         axis('equal') sets the xlim width and ylim height to be to be
             identical.  The longer of the two intervals is chosen
 
+        axis('scaled') changes the size of the subplot such that equal
+          tick mark increments on the x- and y-axis are equal in size
+          (like the 'axis equal' command in matplab). This makes a
+          circle look like a circle, for example
 
        if len(*v)==0, you can pass in xmin, xmax, ymin, ymax as kwargs
        selectively to alter just those limits w/o changing the others.
@@ -539,7 +547,22 @@ def axis(*v, **kwargs):
             ax.set_xlim((xmin, xmin+interval))
             ax.set_ylim((ymin, ymin+interval))
             draw_if_interactive()
-
+        elif s.lower()=='scaled':
+            figwidth,figheight = gcf().get_size_inches()
+            figwidth = 0.8*figwidth; figheight = 0.8*figheight
+            ax = gca()
+            x1,x2 = ax.get_xlim()
+            y1,y2 = ax.get_ylim()
+            plotheight = y2-y1
+            plotwidth = x2-x1
+            if plotheight/plotwidth > figheight/figwidth:  # Plot is higher than figure
+                wfrac = figheight/figwidth * plotwidth/plotheight
+                hfrac = 1.0
+            else:
+                hfrac = figwidth/figheight * plotheight/plotwidth
+                wfrac = 1.0
+            gcf().subplots_adjust( left = 0.1, right = 0.1+wfrac*0.8, bottom = 0.1, top = 0.1+hfrac*0.8 )
+            draw_if_interactive()
         else:
             raise ValueError('Unrecognized string %s to axis; try on or off' % s)
         ax  = gca()
@@ -564,7 +587,6 @@ def axis(*v, **kwargs):
     gca().set_ylim([v[2], v[3]])
     draw_if_interactive()
     return v
-
 def axes(*args, **kwargs):
     """
     Add an axes at positon rect specified by::
@@ -714,7 +736,7 @@ def figtext(*args, **kwargs):
     draw_if_interactive()
     return ret
 if Figure.text.__doc__ is not None:
-    figtext.__doc__ = Figure.text.__doc__
+    figtext.__doc__ = _shift_string(Figure.text.__doc__)
 
 def figimage(*args, **kwargs):
     # allow callers to override the hold state by passing hold=True|False
@@ -723,7 +745,7 @@ def figimage(*args, **kwargs):
     gci._current = ret
     return ret
 if Figure.figimage.__doc__ is not None:
-    figimage.__doc__ = Figure.figimage.__doc__ + """
+    figimage.__doc__ = _shift_string(Figure.figimage.__doc__) + """
 Addition kwargs: hold = [True|False] overrides default hold state"""
 
 def figlegend(handles, labels, loc, **kwargs):
@@ -750,7 +772,7 @@ def savefig(*args, **kwargs):
     fig = gcf()
     return fig.savefig(*args, **kwargs)
 if Figure.savefig.__doc__ is not None:
-    savefig.__doc__ = Figure.savefig.__doc__
+    savefig.__doc__ = _shift_string(Figure.savefig.__doc__)
 
 
 def figure(num=None, # autoincrement if None, else integer from 1-N
@@ -803,7 +825,8 @@ def figure(num=None, # autoincrement if None, else integer from 1-N
         figManager = new_figure_manager(num, figsize, dpi, facecolor, edgecolor, frameon)
         _pylab_helpers.Gcf.set_active(figManager)
         figManager.canvas.figure.number = num
-        
+
+    draw_if_interactive()
     return figManager.canvas.figure
 
 def gca(**kwargs):
@@ -883,7 +906,7 @@ def isinteractive():
 def imread(*args, **kwargs):
     return _imread(*args, **kwargs)
 if _imread.__doc__ is not None:
-    imread.__doc__ = _imread.__doc__
+    imread.__doc__ = _shift_string(_imread.__doc__)
 
 
 def load(fname,comments='%',delimiter=None):
@@ -915,9 +938,9 @@ def load(fname,comments='%',delimiter=None):
 
     comments is the character used to indicate the start of a comment
     in the file
-    
+
     delimiter is a string-like character used to seperate values in the
-    file. If delimiter is unspecified or none, any whitespace string is 
+    file. If delimiter is unspecified or none, any whitespace string is
     a separator.
 
     """
@@ -953,13 +976,13 @@ def load(fname,comments='%',delimiter=None):
 def rc(*args, **kwargs):
     matplotlib.rc(*args, **kwargs)
 if matplotlib.rc.__doc__ is not None:
-    rc.__doc__ =   matplotlib.rc.__doc__
+    rc.__doc__ =  _shift_string(matplotlib.rc.__doc__)
 
 def rcdefaults():
     matplotlib.rcdefaults()
     draw_if_interactive()
 if matplotlib.rcdefaults.__doc__ is not None:
-    rcdefaults.__doc__ =   matplotlib.rcdefaults.__doc__
+    rcdefaults.__doc__ =   _shift_string(matplotlib.rcdefaults.__doc__)
 
 def save(fname, X, fmt='%.18e'):
     """
@@ -1515,14 +1538,14 @@ def subplots_adjust(*args, **kwargs):
       bottom = 0.1   # the bottom of the subplots of the figure
       top = 0.9      # the top of the subplots of the figure
       wspace = 0.2   # the amount of width reserved for blank space between subplots
-      hspace = 0.2   # the amount of height reserved for white space between subplots        
+      hspace = 0.2   # the amount of height reserved for white space between subplots
 
-    The actual defaults are controlled by the rc file    
+    The actual defaults are controlled by the rc file
     """
     fig = gcf()
     fig.subplots_adjust(*args, **kwargs)
     draw_if_interactive()
-    
+
 
 def subplot_tool(targetfig=None):
     """
@@ -1540,14 +1563,14 @@ def subplot_tool(targetfig=None):
         for manager in _pylab_helpers.Gcf._activeQue:
             if manager.canvas.figure==targetfig: break
         else: raise RuntimeError('Could not find manager for targetfig')
-    
+
     toolfig = figure(figsize=(6,3))
     toolfig.subplots_adjust(top=0.9)
     ret =  SubplotTool(targetfig, toolfig)
     rcParams['toolbar'] = tbar
     _pylab_helpers.Gcf.set_active(manager)  # restore the current figure
     return ret
-    
+
 ### The following functions were autogenerated by the boilerplate.py
 ### script.  They are simple wrappers around the Axes methods of the
 
@@ -1565,7 +1588,7 @@ def axhline(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axhline.__doc__ is not None:
@@ -1586,7 +1609,7 @@ def axhspan(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axhspan.__doc__ is not None:
@@ -1607,7 +1630,7 @@ def axvline(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axvline.__doc__ is not None:
@@ -1628,7 +1651,7 @@ def axvspan(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.axvspan.__doc__ is not None:
@@ -1649,7 +1672,7 @@ def bar(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.bar.__doc__ is not None:
@@ -1670,7 +1693,7 @@ def barh(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.barh.__doc__ is not None:
@@ -1691,7 +1714,7 @@ def boxplot(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.boxplot.__doc__ is not None:
@@ -1712,7 +1735,7 @@ def cohere(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.cohere.__doc__ is not None:
@@ -1796,7 +1819,7 @@ def csd(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.csd.__doc__ is not None:
@@ -1817,7 +1840,7 @@ def errorbar(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.errorbar.__doc__ is not None:
@@ -1838,7 +1861,7 @@ def fill(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.fill.__doc__ is not None:
@@ -1859,7 +1882,7 @@ def hist(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.hist.__doc__ is not None:
@@ -1880,7 +1903,7 @@ def hlines(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.hlines.__doc__ is not None:
@@ -1922,7 +1945,7 @@ def loglog(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.loglog.__doc__ is not None:
@@ -1964,7 +1987,7 @@ def pcolor_classic(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.pcolor_classic.__doc__ is not None:
@@ -1985,7 +2008,7 @@ def pie(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.pie.__doc__ is not None:
@@ -2006,7 +2029,7 @@ def plot(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.plot.__doc__ is not None:
@@ -2027,7 +2050,7 @@ def plot_date(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.plot_date.__doc__ is not None:
@@ -2048,7 +2071,7 @@ def psd(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.psd.__doc__ is not None:
@@ -2090,7 +2113,7 @@ def scatter_classic(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.scatter_classic.__doc__ is not None:
@@ -2111,7 +2134,7 @@ def semilogx(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.semilogx.__doc__ is not None:
@@ -2132,7 +2155,7 @@ def semilogy(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.semilogy.__doc__ is not None:
@@ -2174,7 +2197,7 @@ def spy(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.spy.__doc__ is not None:
@@ -2216,7 +2239,7 @@ def stem(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.stem.__doc__ is not None:
@@ -2237,7 +2260,7 @@ def vlines(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.vlines.__doc__ is not None:
@@ -2258,7 +2281,7 @@ def quiver(*args, **kwargs):
     except:
         hold(b)
         raise
-    
+
     hold(b)
     return ret
 if Axes.quiver.__doc__ is not None:
