@@ -11,22 +11,20 @@ plot(t, s)
 grid(True)
 
 # matlab handle graphics style
-xticklines = get(gca(), 'xticklines')
-yticklines = get(gca(), 'yticklines')
-xgridlines = get(gca(), 'xgridlines')
-ygridlines = get(gca(), 'ygridlines')
-xticklabels = get(gca(), 'xticklabels')
-yticklabels = get(gca(), 'yticklabels')
+xticklines = getp(gca(), 'xticklines')
+yticklines = getp(gca(), 'yticklines')
+xgridlines = getp(gca(), 'xgridlines')
+ygridlines = getp(gca(), 'ygridlines')
+xticklabels = getp(gca(), 'xticklabels')
+yticklabels = getp(gca(), 'yticklabels')
 
-set(xticklines, 'linewidth', 3)
-set(yticklines, 'linewidth', 3)
-set(xgridlines, 'linestyle', '-')
-set(ygridlines, 'linestyle', '-')
-set(yticklabels, 'color', 'r', fontsize='medium')
-set(xticklabels, 'color', 'r', fontsize='medium')
+setp(xticklines, 'linewidth', 3)
+setp(yticklines, 'linewidth', 3)
+setp(xgridlines, 'linestyle', '-')
+setp(ygridlines, 'linestyle', '-')
+setp(yticklabels, 'color', 'r', fontsize='medium')
+setp(xticklabels, 'color', 'r', fontsize='medium')
 
-# keyword args are legal too
-#set(xticklabels, color='r', fontsize='medium')
 
 #savefig('axprops_demo')
 show()
