@@ -8,5 +8,5 @@ from pylab import *
 
 plot([1,2,3])
 
-savefig(sys.stdout)
+#savefig(sys.stdout)
 show()
